/* @(#)e_jn.c 5.1 93/09/24 */
/*
 * ====================================================
 * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
 *
 * Developed at SunPro, a Sun Microsystems, Inc. business.
 * Permission to use, copy, modify, and distribute this
 * software is freely granted, provided that this notice
 * is preserved.
 * ====================================================
 */

/*
 * jn(n, x), yn(n, x)
 * floating point Bessel's function of the 1st and 2nd kind
 * of order n
 *
 * Special cases:
 *	y0(0)=y1(0)=yn(n,0) = -inf with division by zero signal;
 *	y0(-ve)=y1(-ve)=yn(n,-ve) are NaN with invalid signal.
 * Note 2. About jn(n,x), yn(n,x)
 *	For n=0, j0(x) is called,
 *	for n=1, j1(x) is called,
 *	for n<x, forward recursion us used starting
 *	from values of j0(x) and j1(x).
 *	for n>x, a continued fraction approximation to
 *	j(n,x)/j(n-1,x) is evaluated and then backward
 *	recursion is used starting from a supposed value
 *	for j(n,x). The resulting value of j(0,x) is
 *	compared with the actual value to correct the
 *	supposed value of j(n,x).
 *
 *	yn(n,x) is similar in all respects, except
 *	that forward recursion is used for all
 *	values of n>1.
 *
 */

#include "fdlibm.h"

#ifdef _NEED_FLOAT64

static const __float64 invsqrtpi = _F_64(5.64189583547756279280e-01), /* 0x3FE20DD7, 0x50429B6D */
    two = _F_64(2.00000000000000000000e+00),                          /* 0x40000000, 0x00000000 */
    one = _F_64(1.00000000000000000000e+00);                          /* 0x3FF00000, 0x00000000 */

static const __float64 zero = _F_64(0.00000000000000000000e+00);

__float64
jn64(int n, __float64 x)
{
    __int32_t i, hx, ix, lx, sgn;
    __float64 a, b, temp, di;
    __float64 z, w;

    if (isnan(x))
        return x + x;

    if (isinf(x))
        return _F_64(0.0);

    /* J(-n,x) = (-1)^n * J(n, x), J(n, -x) = (-1)^n * J(n, x)
     * Thus, J(-n,x) = J(n,-x)
     */
    EXTRACT_WORDS(hx, lx, x);
    ix = 0x7fffffff & hx;

    if (n < 0) {
        n = -n;
        x = -x;
        hx ^= 0x80000000;
    }
    if (n == 0)
        return (j064(x));
    if (n == 1)
        return (j164(x));
    sgn = (n & 1) & (hx >> 31); /* even n -- 0, odd n -- sign(x) */
    x = fabs64(x);
    if ((ix | lx) == 0 || ix >= 0x7ff00000) /* if x is 0 or inf */
        b = zero;
    else if ((__float64)n <= x) {
        /* Safe to use J(n+1,x)=2n/x *J(n,x)-J(n-1,x) */
        if (ix >= 0x52D00000) { /* x > 2**302 */
            /* (x >> n**2)
             *	    Jn(x) = cos(x-(2n+1)*pi/4)*sqrt(2/x*pi)
             *	    Yn(x) = sin(x-(2n+1)*pi/4)*sqrt(2/x*pi)
             *	    Let s=sin(x), c=cos(x),
             *		xn=x-(2n+1)*pi/4, sqt2 = sqrt(2),then
             *
             *		   n	sin(xn)*sqt2	cos(xn)*sqt2
             *		----------------------------------
             *		   0	 s-c		 c+s
             *		   1	-s-c 		-c+s
             *		   2	-s+c		-c-s
             *		   3	 s+c		 c-s
             */
            switch (n & 3) {
            case 0:
            default:
                temp = cos64(x) + sin64(x);
                break;
            case 1:
                temp = -cos64(x) + sin64(x);
                break;
            case 2:
                temp = -cos64(x) - sin64(x);
                break;
            case 3:
                temp = cos64(x) - sin64(x);
                break;
            }
            b = invsqrtpi * temp / sqrt64(x);
        } else {
            a = j064(x);
            b = j164(x);
            for (i = 1; i < n; i++) {
                temp = b;
                b = b * ((__float64)(i + i) / x) - a; /* avoid underflow */
                a = temp;
            }
        }
    } else {
        if (ix < 0x3e100000) { /* x < 2**-29 */
            /* x is tiny, return the first Taylor expansion of J(n,x)
             * J(n,x) = 1/n!*(x/2)^n  - ...
             */
            if (n > 33) /* underflow */
                b = zero;
            else {
                temp = x * _F_64(0.5);
                b = temp;
                for (a = one, i = 2; i <= n; i++) {
                    a *= (__float64)i; /* a = n! */
                    b *= temp;         /* b = (x/2)^n */
                }
                b = b / a;
            }
        } else {
            /* use backward recurrence */
            /* 			x      x^2      x^2
             *  J(n,x)/J(n-1,x) =  ----   ------   ------   .....
             *			2n  - 2(n+1) - 2(n+2)
             *
             * 			1      1        1
             *  (for large x)   =  ----  ------   ------   .....
             *			2n   2(n+1)   2(n+2)
             *			-- - ------ - ------ -
             *			 x     x         x
             *
             * Let w = 2n/x and h=2/x, then the above quotient
             * is equal to the continued fraction:
             *		    1
             *	= -----------------------
             *		       1
             *	   w - -----------------
             *			  1
             * 	        w+h - ---------
             *		       w+2h - ...
             *
             * To determine how many terms needed, let
             * Q(0) = w, Q(1) = w(w+h) - 1,
             * Q(k) = (w+k*h)*Q(k-1) - Q(k-2),
             * When Q(k) > 1e4	good for single
             * When Q(k) > 1e9	good for double
             * When Q(k) > 1e17	good for quadruple
             */
            /* determine k */
            __float64 t, v;
            __float64 q0, q1, h, tmp;
            __int32_t k, m;
            w = (n + n) / (__float64)x;
            h = _F_64(2.0) / (__float64)x;
            q0 = w;
            z = w + h;
            q1 = w * z - _F_64(1.0);
            k = 1;
            while (q1 < _F_64(1.0e9)) {
                k += 1;
                z += h;
                tmp = z * q1 - q0;
                q0 = q1;
                q1 = tmp;
            }
            m = n + n;
            for (t = zero, i = 2 * (n + k); i >= m; i -= 2)
                t = one / (i / x - t);
            a = t;
            b = one;
            /*  estimate log((2/x)^n*n!) = n*log(2/x)+n*ln(n)
             *  Hence, if n*(log(2n/x)) > ...
             *  single 8.8722839355e+01
             *  double 7.09782712893383973096e+02
             *  long double 1.1356523406294143949491931077970765006170e+04
             *  then recurrent value may overflow and the result is
             *  likely underflow to zero
             */
            tmp = n;
            v = two / x;
            tmp = tmp * log64(fabs64(v * tmp));
            if (tmp < _F_64(7.09782712893383973096e+02)) {
                for (i = n - 1, di = (__float64)(i + i); i > 0; i--) {
                    temp = b;
                    b *= di;
                    b = b / x - a;
                    a = temp;
                    di -= two;
                }
            } else {
                for (i = n - 1, di = (__float64)(i + i); i > 0; i--) {
                    temp = b;
                    b *= di;
                    b = b / x - a;
                    a = temp;
                    di -= two;
                    /* scale b to avoid spurious overflow */
                    if (b > _F_64(1e100)) {
                        a /= b;
                        t /= b;
                        b = one;
                    }
                }
            }
            b = (t * j064(x) / b);
        }
    }
    if (sgn == 1)
        return -b;
    else
        return b;
}

_MATH_ALIAS_d_id(jn)

    __float64 yn64(int n, __float64 x)
{
    __int32_t i, hx, ix, lx;
    __int32_t sign;
    __float64 a, b, temp;

    EXTRACT_WORDS(hx, lx, x);
    ix = 0x7fffffff & hx;
    /* if Y(n,NaN) is NaN */

    if ((ix | lx) == 0)
        return __math_divzero(1);

    if (isnan(x))
        return x + x;

    if (hx < 0)
        return __math_invalid(x);

    if (ix == 0x7ff00000)
        return _F_64(0.0);

    sign = 1;
    if (n < 0) {
        n = -n;
        sign = 1 - ((n & 1) << 1);
    }
    if (n == 0)
        return (y064(x));
    if (n == 1)
        return (sign * y164(x));

    if (ix >= 0x52D00000) { /* x > 2**302 */
        /* (x >> n**2)
         *	    Jn(x) = cos(x-(2n+1)*pi/4)*sqrt(2/x*pi)
         *	    Yn(x) = sin(x-(2n+1)*pi/4)*sqrt(2/x*pi)
         *	    Let s=sin(x), c=cos(x),
         *		xn=x-(2n+1)*pi/4, sqt2 = sqrt(2),then
         *
         *		   n	sin(xn)*sqt2	cos(xn)*sqt2
         *		----------------------------------
         *		   0	 s-c		 c+s
         *		   1	-s-c 		-c+s
         *		   2	-s+c		-c-s
         *		   3	 s+c		 c-s
         */
        switch (n & 3) {
        case 0:
        default:
            temp = sin64(x) - cos64(x);
            break;
        case 1:
            temp = -sin64(x) - cos64(x);
            break;
        case 2:
            temp = -sin64(x) + cos64(x);
            break;
        case 3:
            temp = sin64(x) + cos64(x);
            break;
        }
        b = invsqrtpi * temp / sqrt64(x);
    } else {
        __uint32_t high;
        a = y064(x);
        b = y164(x);
        /* quit if b is -inf */
        GET_HIGH_WORD(high, b);
        for (i = 1; i < n && high != 0xfff00000; i++) {
            temp = b;
            b = ((__float64)(i + i) / x) * b - a;
            GET_HIGH_WORD(high, b);
            a = temp;
        }
    }
    if (sign > 0)
        return b;
    else
        return -b;
}

_MATH_ALIAS_d_id(yn)

#endif /* _NEED_FLOAT64 */
