/*
Copyright (c) 1994 Cygnus Support.
All rights reserved.

Redistribution and use in source and binary forms are permitted
provided that the above copyright notice and this paragraph are
duplicated in all such forms and that any documentation,
and/or other materials related to such
distribution and use acknowledge that the software was developed
at Cygnus Support, Inc.  Cygnus Support, Inc. may not be used to
endorse or promote products derived from this software without
specific prior written permission.
THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
/*
FUNCTION
        <<memmove>>---move possibly overlapping memory

INDEX
        memmove

SYNOPSIS
        #include <string.h>
        void *memmove(void *<[dst]>, const void *<[src]>, size_t <[length]>);

DESCRIPTION
        This function moves <[length]> characters from the block of
        memory starting at <<*<[src]>>> to the memory starting at
        <<*<[dst]>>>. <<memmove>> reproduces the characters correctly
        at <<*<[dst]>>> even if the two areas overlap.


RETURNS
        The function returns <[dst]> as passed.

PORTABILITY
<<memmove>> is ANSI C.

<<memmove>> requires no supporting OS subroutines.

QUICKREF
        memmove ansi pure
*/

#include <string.h>
#include <stddef.h>
#include <limits.h>
#include <stdint.h>
#include "local.h"

#undef memmove

/*SUPPRESS 20*/
void * __no_builtin
memmove(void *dst_void, const void *src_void, size_t length)
{
#if defined(__PREFER_SIZE_OVER_SPEED) || defined(__OPTIMIZE_SIZE__)
    char       *dst = dst_void;
    const char *src = src_void;

    if (src < dst && dst < src + length) {
        /* Have to copy backwards */
        src += length;
        dst += length;
        while (length--) {
            *--dst = *--src;
        }
    } else {
        while (length--) {
            *dst++ = *src++;
        }
    }

    return dst_void;
#else
    char       *dst = dst_void;
    const char *src = src_void;
    long       *aligned_dst;
    const long *aligned_src;

    if (src < dst && dst < src + length) {
        /* Destructive overlap...have to copy backwards */
        src += length;
        dst += length;

        if (!TOO_SMALL_LITTLE_BLOCK(length) && !UNALIGNED_X_Y(src, dst)) {
            aligned_dst = (long *)dst;
            aligned_src = (long *)src;

            /* Copy one long word at a time if possible.  */
            while (!TOO_SMALL_LITTLE_BLOCK(length)) {
                *--aligned_dst = *--aligned_src;
                length -= LITTLE_BLOCK_SIZE;
            }

            /* Pick up any residual with a byte copier.  */
            dst = (char *)aligned_dst;
            src = (char *)aligned_src;
        }

        while (length--) {
            *--dst = *--src;
        }
    } else {
        /* Use optimizing algorithm for a non-destructive copy to closely
           match memcpy. If the size is small or either SRC or DST is unaligned,
           then punt into the byte copy loop.  This should be rare.  */
        if (!TOO_SMALL_LITTLE_BLOCK(length) && !UNALIGNED_X_Y(src, dst)) {
            aligned_dst = (long *)dst;
            aligned_src = (long *)src;

            /* Copy 4X long words at a time if possible.  */
            while (!TOO_SMALL_BIG_BLOCK(length)) {
                *aligned_dst++ = *aligned_src++;
                *aligned_dst++ = *aligned_src++;
                *aligned_dst++ = *aligned_src++;
                *aligned_dst++ = *aligned_src++;
                length -= BIG_BLOCK_SIZE;
            }

            /* Copy one long word at a time if possible.  */
            while (!TOO_SMALL_LITTLE_BLOCK(length)) {
                *aligned_dst++ = *aligned_src++;
                length -= LITTLE_BLOCK_SIZE;
            }

            /* Pick up any residual with a byte copier.  */
            dst = (char *)aligned_dst;
            src = (char *)aligned_src;
        }

        while (length--) {
            *dst++ = *src++;
        }
    }

    return dst_void;
#endif /* not __PREFER_SIZE_OVER_SPEED */
}
