/*
Copyright (c) 1990 The Regents of the University of California.
All rights reserved.

Redistribution and use in source and binary forms are permitted
provided that the above copyright notice and this paragraph are
duplicated in all such forms and that any documentation,
and/or other materials related to such
distribution and use acknowledge that the software was developed
by the University of California, Berkeley.  The name of the
University may not be used to endorse or promote products derived
from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
#include <picolibc.h>

	.global	___lshrhi15
___lshrhi15:
	lsr	a

	.global	___lshrhi14
___lshrhi14:		
	lsr	a	
	.global	___lshrhi13
___lshrhi13:	
	lsr	a	
	.global	___lshrhi12
___lshrhi12:	
	lsr	a	
	.global	___lshrhi11
___lshrhi11:	
	lsr	a	
	.global	___lshrhi10
___lshrhi10:	
	lsr	a	
	.global	___lshrhi9
___lshrhi9:	
	lsr	a	
	.global	___lshrhi8
___lshrhi8:	
	lsr	a	
	.global	___lshrhi7
___lshrhi7:	
	lsr	a	
	.global	___lshrhi6
___lshrhi6:	
	lsr	a	
	.global	___lshrhi5
___lshrhi5:	
	lsr	a	
	.global	___lshrhi4
___lshrhi4:	
	lsr	a	
	.global	___lshrhi3
___lshrhi3:	
	lsr	a	
	.global	___lshrhi2
___lshrhi2:
	lsr	a
	lsr	a
	rtl

