/* Copyright (c) 2004 Jeff Johnston  <jjohnstn@redhat.com> */
/* ARM configuration file; HZ is 100 rather than the default 60 */

#ifndef _MACHINE_PARAM_H
#define _MACHINE_PARAM_H

#define HZ (100)

#endif
