/* Copyright (c) 2010 Yaakov Selkowitz <yselkowi@redhat.com> */
/* BSD predecessor of POSIX.1 <dirent.h> and struct dirent */

#ifndef _SYS_DIR_H_
#define _SYS_DIR_H_

#include <dirent.h>

#define direct dirent

#endif /*_SYS_DIR_H_*/
