/*-
 * Copyright (c) 2007-2008 David Schultz <das@FreeBSD.ORG>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#define _GNU_SOURCE
#include <complex.h>
#include <float.h>
#include <math.h>
#include <stdbool.h>

#ifdef __HAVE_LONG_DOUBLE

/*
 * gcc doesn't implement complex multiplication or division correctly,
 * so we need to handle infinities specially. We turn on this pragma to
 * notify conforming c99 compilers that the fast-but-incorrect code that
 * gcc generates is acceptable, since the special cases have already been
 * handled.
 */
// #pragma	STDC CX_LIMITED_RANGE	ON

/* We risk spurious overflow for components >= LDBL_MAX / (1 + sqrt(2)). */
#define THRESH       (LDBL_MAX / 2.414213562373095048801688724209698L)

#define cpackl(r, i) ((r) + (i) * (long double complex)I)

long double complex
csqrtl(long double complex z)
{
    long double complex result;
    long double         a, b;
    long double         t;
    bool                scale;

    a = creall(z);
    b = cimagl(z);

    /* Handle special cases. */
    if (z == 0.0L)
        return (cpackl((long double)0.0L, b));
    if (isinf(b))
        return (cpackl((long double)INFINITY, b));
    if (isnan(a)) {
        t = (b - b) / (b - b); /* raise invalid if b is not a NaN */
        return (cpackl(a, t)); /* return NaN + NaN i */
    }
    if (isinf(a)) {
        /*
         * csqrt(inf + NaN i)  = inf +  NaN i
         * csqrt(inf + y i)    = inf +  0 i
         * csqrt(-inf + NaN i) = NaN +- inf i
         * csqrt(-inf + y i)   = 0   +  inf i
         */
        if (signbit(a))
            return (cpackl(fabsl(b - b), copysignl(a, b)));
        else
            return (cpackl(a, copysignl(b - b, b)));
    }
    /*
     * The remaining special case (b is NaN) is handled just fine by
     * the normal code path below.
     */

    /* Scale to avoid overflow. */
    if (fabsl(a) >= THRESH || fabsl(b) >= THRESH) {
        a *= 0.25L;
        b *= 0.25L;
        scale = true;
    } else {
        scale = false;
    }

    /* Algorithm 312, CACM vol 10, Oct 1967. */
    if (a >= 0L) {
        t = sqrtl((a + hypotl(a, b)) * 0.5L);
        result = cpackl(t, b / (2.0L * t));
    } else {
        t = sqrtl((-a + hypotl(a, b)) * 0.5L);
        result = cpackl(fabsl(b) / (2.0L * t), copysignl(t, b));
    }

    /* Rescale. */
    if (scale)
        return (result * 2.0L);
    else
        return (result);
}

#endif /* __HAVE_LONG_DOUBLE */
