/*
 *  Written by Joel Sherrill <joel@OARcorp.com>.
 *
 *  COPYRIGHT (c) 1989-2010.
 *  On-Line Applications Research Corporation (OAR).
 *
 *  Permission to use, copy, modify, and distribute this software for any
 *  purpose without fee is hereby granted, provided that this entire notice
 *  is included in all copies of any software which is or includes a copy
 *  or modification of this software.
 *
 *  THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
 *  WARRANTY.  IN PARTICULAR,  THE AUTHOR MAKES NO REPRESENTATION
 *  OR WARRANTY OF ANY KIND CONCERNING THE MERCHANTABILITY OF THIS
 *  SOFTWARE OR ITS FITNESS FOR ANY PARTICULAR PURPOSE.
 *
 *  $Id$
 */

#ifndef _SYS_SCHED_H_
#define _SYS_SCHED_H_

#include <sys/cdefs.h>
#include <sys/_types.h>
#include <sys/_timespec.h>

_BEGIN_STD_C

/* Scheduling Policies */
/* Open Group Specifications Issue 6 */
#define SCHED_OTHER 0

#define SCHED_FIFO  1
#define SCHED_RR    2

#if defined(_POSIX_SPORADIC_SERVER)
#define SCHED_SPORADIC 4
#endif

#if __GNU_VISIBLE
#define SCHED_IDLE  5
#define SCHED_BATCH 6

/* Flag to drop realtime policies and negative nice values on fork(). */
#define SCHED_RESET_ON_FORK 0x40000000
#endif

/* Scheduling Parameters */
/* Open Group Specifications Issue 6 */

struct sched_param {
    int sched_priority; /* Process execution scheduling priority */

#if defined(_POSIX_SPORADIC_SERVER) || defined(_POSIX_THREAD_SPORADIC_SERVER)
    int             sched_ss_low_priority; /* Low scheduling priority for sporadic */
                                           /*   server */
    struct timespec sched_ss_repl_period;
    /* Replenishment period for sporadic server */
    struct timespec sched_ss_init_budget;
    /* Initial budget for sporadic server */
    int             sched_ss_max_repl; /* Maximum pending replenishments for */
                                       /* sporadic server */
#endif
};

_END_STD_C

#endif
/* end of include file */
